#!/usr/bin/env python3

__author__ = "Yxzh"

import pygame
import sys
from pygame.locals import *
from pygame.color import THECOLORS
import pygame.font


class UI(object):
	def __init__(self, visual_mode = True, fps = 60):
		"""
		初始化游戏界面。
		:param visual_mode: 可视化模式
		:param fps: 游戏帧率
					根据计算性能，会有实际帧率低于设定帧率的情况。
		"""
		self.PLAYGROUND_WIDTH = 20
		self.PLAYGROUND_HEIGHT = 20  # 游戏区域大小
		self.INFOAREA_WIDTH = 100
		self.INFOAREA_HEIGHT = 200  # 信息区域大小
		self.visual_mode = visual_mode  # 显示模式 0为默认 1为图像模式
		WINDOW_TITLE = "Snacky"  # 屏幕标题
		allowed_event = [pygame.KEYDOWN, pygame.QUIT]  # 事件列表
		pygame.init()  # 初始化pygame
		pygame.display.set_caption(WINDOW_TITLE)  # 窗口标题
		self.fps_clock = pygame.time.Clock()  # 创建FPS时钟对象
		pygame.event.set_allowed(allowed_event)  # 设置事件过滤
		self.is_pause = False
		if self.visual_mode:
			self.s_screen = pygame.display.set_mode(
				(self.PLAYGROUND_WIDTH * 20 + self.INFOAREA_WIDTH, self.PLAYGROUND_HEIGHT * 10), 0,
				32)
		else:
			self.s_screen = pygame.display.set_mode(
				(self.PLAYGROUND_WIDTH * 10 + self.INFOAREA_WIDTH, self.PLAYGROUND_HEIGHT * 10), 0,
				32)
		self.s_infoarea = pygame.Surface((self.PLAYGROUND_WIDTH * 5, self.PLAYGROUND_HEIGHT * 10), 0, 32)  # 信息区域Surface
		self.s_pltarea = pygame.Surface((self.PLAYGROUND_WIDTH * 10, self.PLAYGROUND_HEIGHT * 10), 0, 32)  # 图表区域Surface
		self.si_snake = pygame.image.load("images/snake.png").convert_alpha()  # 加载图片
		self.si_food = pygame.image.load("images/Pineapple.png").convert_alpha()
		self.si_deadsnake = pygame.image.load("images/dead_snake.png").convert_alpha()
		self.si_gamestart = pygame.image.load("images/game_start.png").convert_alpha()
		self.si_bomb = pygame.image.load("images/bm.png").convert_alpha()
		pygame.font.init()  # 初始化字体
		f_arial = pygame.font.SysFont("arial", 24)  # 加载字体
		f_optima = pygame.font.SysFont("optima", 35)
		f_small_arial = pygame.font.SysFont("arial", 12, 1)
		self.sf_arial_gameover = f_arial.render("GAMEOVER!", 1, THECOLORS["black"])  # 渲染文字
		self.sf_arial_scoreboad = f_arial.render("SCOREBOARD", 1, THECOLORS["black"])
		self.sf_small_arial_restart = f_small_arial.render("Press 'R' to restart.", 1, THECOLORS["black"])
		self.sf_small_arial_scoreboard = f_small_arial.render("Press 'S' to open Scoreboard.", 1, THECOLORS["black"])
		self.sf_small_arial_gamestart = f_small_arial.render("SPACE to start and 'Q' to exit.", 1, THECOLORS["black"])
		self.sf_small_arial_back = f_small_arial.render("Press 'R' to return.", 1, THECOLORS["black"])
		self.sf_small_arial_score = f_small_arial.render("Score: ", 1, THECOLORS["red"])
		self.sf_optima_caption = f_optima.render("SNACKY", 1, THECOLORS["orange"])
		self.sf_small_arial_level = f_small_arial.render("level: ", 1, THECOLORS["black"])
		self.sf_small_arial_current_fps = f_small_arial.render("fps: ", 1, THECOLORS["black"])
		self.sf_small_arial_ate = f_small_arial.render("ate: ", 1, THECOLORS["black"])
		self.sf_small_arial_position = f_small_arial.render("pos: ", 1, THECOLORS["black"])
		self.sf_small_arial_botton = f_small_arial.render("btnreg: ", 1, THECOLORS["black"])
		self.sf_small_arial_direction = f_small_arial.render("direction: ", 1, THECOLORS["black"])
		self.sf_small_arial_dot = f_small_arial.render(".", 1, THECOLORS["black"])
		self.Lsf_small_arial_direction = {
			"W": f_small_arial.render("W", 1, THECOLORS["black"]),
			"S": f_small_arial.render("S", 1, THECOLORS["black"]),
			"A": f_small_arial.render("A", 1, THECOLORS["black"]),
			"D": f_small_arial.render("D", 1, THECOLORS["black"]),
		}
		self.Lsf_small_arial_numbers_black = []
		for i in range(0, 10):
			self.Lsf_small_arial_numbers_black.append(f_small_arial.render(str(i), 1, THECOLORS["black"]))
		self.fps = fps  # 帧率
	
	def show(self, game, agent):
		"""
		显示图形界面。
		:param game: 游戏物理引擎类
		:param agent: 决策逻辑类
		"""
		self.start()  # 开始游戏画面
		while True:  # 屏幕循环
			for event in pygame.event.get():  # 事件循环
				if event.type == pygame.QUIT or event.type == KEYDOWN and event.key == K_q:  # 退出事件
					pygame.quit()
					sys.exit()
				if event.type == KEYDOWN and event.key == K_e:
					agent.custom_function(game)
				if event.type == KEYDOWN and event.key == K_f:
					if self.visual_mode:
						self.s_screen = pygame.display.set_mode(
							(self.PLAYGROUND_WIDTH * 10 + self.INFOAREA_WIDTH, self.PLAYGROUND_HEIGHT * 10), 0,
							32)
						self.visual_mode = False
					else:
						self.s_screen = pygame.display.set_mode(
							(self.PLAYGROUND_WIDTH * 20 + self.INFOAREA_WIDTH, self.PLAYGROUND_HEIGHT * 10), 0,
							32)
						self.visual_mode = True
						
				if event.type == KEYDOWN and event.key == K_p:
					self.pause()
			self.s_screen.fill(THECOLORS["white"])  # 填充白屏
			game.next(agent.get_next_direction(game))  # 获取下一步方向
			if self.visual_mode:  #可视化模式
				strategy_max = agent.strategy.max()
				strategy_min = agent.strategy.min()
				for i in range(0, 20):
					for j in range(0, 20):
						bright = (agent.strategy[i][j] - strategy_min) / (strategy_max - strategy_min) * 255
						pygame.draw.rect(self.s_screen, [bright] * 3,
						                 [i * 10 + 300, j * 10, 10, 10], 0)
						pygame.draw.line(self.s_screen,THECOLORS["black"], (300, 0), (300, 200), 2)
			self.s_screen.blit(self.si_food, (game.food_pos[0] * 10 - 4, game.food_pos[1] * 10 - 5))  # 填充食物图片
			for i in game.bombs:  # 填充炸弹图片
				self.s_screen.blit(self.si_bomb, [x * 10 - 2 for x in i])
			for i in game.snakes:  # 填充蛇图片
				self.s_screen.blit(self.si_snake, [x * 10 for x in i])
			if game.deathflag:  # 死亡判定
				self.f_gameover(game.ate)  # 游戏结束画面
				game.reset()
			self.s_infoarea.fill(THECOLORS["gray"])  # 开始填充信息区域各种信息
			pygame.draw.line(self.s_infoarea, THECOLORS["black"], (0, 0), (0, self.INFOAREA_HEIGHT), 3)  # 分隔线
			self.s_infoarea.blit(self.sf_small_arial_level, (10, 5))  # 难度信息提示
			self.s_infoarea.blit(self.sf_small_arial_ate, (10, 20))  # 食物数量信息
			self.s_infoarea.blit(self.sf_small_arial_position, (10, 35))  # 位置信息
			self.s_infoarea.blit(self.sf_small_arial_direction, (10, 65))  # 方向信息
			self.s_infoarea.blit(self.sf_small_arial_botton, (10, 80))  # 按键寄存信息 （实时刷新）
			self.s_infoarea.blit(self.sf_small_arial_current_fps, (10, 95))  # FPS（实时刷新）
			self.s_infoarea.blit(self.sf_small_arial_dot, (65, 5))
			self.f_show_number(self.s_infoarea, game.ate, (65, 20))
			self.f_show_number(self.s_infoarea, game.head_pos[0], (65, 35))
			self.f_show_number(self.s_infoarea, game.head_pos[1], (65, 50))
			current_fps = self.fps_clock.get_fps()  # 获取实时FPS
			self.f_show_number(self.s_infoarea, current_fps, (65, 95))  # 填充FPS
			self.s_infoarea.blit(self.Lsf_small_arial_direction[game.direction], (65, 65))
			self.s_infoarea.blit(self.Lsf_small_arial_direction[game.direction], (65, 80))  # 填充按键寄存
			self.s_screen.blit(self.s_infoarea, (self.PLAYGROUND_WIDTH * 10, 0))  # 将实时填充后的信息Surface填充至屏幕Surface
			pygame.display.flip()  # 将图像内存缓冲刷新至屏幕
			self.fps_clock.tick(self.fps)  # FPS等待时钟
			if self.is_pause:
				self.pause()
	
	def f_show_number(self, surface, number, position):
		"""
		在目标Surface上填充数字
		:param surface: 目标Surface
		:param number: 数字
		:param position: 位置
		"""
		
		surface.blit(self.Lsf_small_arial_numbers_black[int(number / 100)], (position[0], position[1]))
		surface.blit(self.Lsf_small_arial_numbers_black[int((number % 100) / 10)], (position[0] + 7, position[1]))
		surface.blit(self.Lsf_small_arial_numbers_black[int((number % 100) % 10)], (position[0] + 14, position[1]))
		return
	
	def start(self):
		"""
		游戏开始画面
		"""
		
		while True:
			for event in pygame.event.get():
				if event.type == pygame.QUIT or (event.type == KEYDOWN and event.key == K_q):  # 退出事件
					pygame.quit()
					sys.exit()
				if event.type == pygame.KEYDOWN and event.key == K_s:  # 进入计分板画面
					self.f_scoreboard()
				if event.type == pygame.KEYDOWN and event.key == K_SPACE:  # 进入main()函数
					return
			self.s_screen.fill(THECOLORS["white"])
			self.s_screen.blit(self.sf_optima_caption, (81, 3))
			self.s_screen.blit(self.si_gamestart, (93, 90))
			self.s_screen.blit(self.sf_small_arial_gamestart, (62, 52))
			self.s_screen.blit(self.sf_small_arial_scoreboard, (62, 70))
			pygame.display.flip()
			self.fps_clock.tick(5)
	
	def f_gameover(self, ate):
		"""
		游戏结束画面
		:param ate: 食物计数
		"""
		
		fi_score = open("score.s", "a+")  # 将分数写入分数文件 不存在就新建
		fi_score.write(str(ate) + "\n")
		fi_score.close()  # 关闭文件IO流
		while True:
			for event in pygame.event.get():
				if event.type == pygame.QUIT or event.type == KEYDOWN and event.key == K_q:  # 退出事件
					pygame.quit()
					sys.exit()
				if event.type == pygame.KEYDOWN and event.key == K_s:  # 进入计分板画面
					self.f_scoreboard()
				if event.type == pygame.KEYDOWN and event.key == K_r:  # 重新开始
					return
			self.s_screen.fill(THECOLORS["white"])
			self.s_screen.blit(self.sf_arial_gameover, (72, 4))
			self.s_screen.blit(self.sf_small_arial_score, (100, 35))
			self.s_screen.blit(self.sf_small_arial_restart, (92, 50))
			self.s_screen.blit(self.sf_small_arial_scoreboard, (62, 65))
			self.s_screen.blit(self.si_deadsnake, (80, 75))
			self.f_show_number(self.s_screen, ate, (150, 35))  # 填充该局得分
			pygame.display.flip()
			self.fps_clock.tick(5)
	
	def f_scoreboard(self):  # 计分板画面
		"""
		计分板画面
		"""
		
		fi_score = open("score.s", "r")  # 读取分数文件
		scores = []
		for i in fi_score.readlines():  # 转换为int
			scores.append(int(i))
		fi_score.close()
		scores.sort(reverse = True)  # 倒序排列
		while True:
			for event in pygame.event.get():
				if event.type == pygame.QUIT or event.type == KEYDOWN and event.key == K_q:
					pygame.quit()
					sys.exit()
				if event.type == pygame.KEYDOWN and event.key == K_r:  # 重新开始
					return
			self.s_screen.fill(THECOLORS["white"])
			self.s_screen.blit(self.sf_arial_scoreboad, (65, 4))
			self.s_screen.blit(self.sf_small_arial_back, (100, 40))
			for i in range(0, len(scores)):  # 填充排行榜
				self.s_screen.blit(self.Lsf_small_arial_numbers_black[i + 1], (130, 60 + 15 * i))
				self.s_screen.blit(self.sf_small_arial_dot, (137, 60 + 15 * i))
				self.f_show_number(self.s_screen, scores[i], (150, 60 + 15 * i))
				if i > 7:
					break
			pygame.display.flip()
			self.fps_clock.tick(5)

	def pause(self):  # 暂停
		"""
		暂停游戏，方便查看当前状态。
		"""
		while True:
			for event in pygame.event.get():
				if event.type == pygame.QUIT or event.type == KEYDOWN and event.key == K_q:
					pygame.quit()
					sys.exit()
				if event.type == pygame.KEYDOWN and event.key == K_p:  # 继续
					self.is_pause = False
					return
				if event.type == pygame.KEYDOWN and event.key == K_n:
					self.is_pause = True
					return


#!/usr/bin/env python3

__author__ = "Yxzh"

from random import randint
import numpy as np


PLAYGROUND_WIDTH = 20
PLAYGROUND_HEIGHT = 20  # 游戏区域大小

class Game(object):
	def __init__(self, bomb = 0):
		"""
		初始化游戏
		:param bomb: 地图中炸弹数量
		"""
		self.bomb_number = bomb  # 炸弹个数
		self.head_pos = [0, 0]  # 蛇头位置
		self.direction = "S"  # 上一步蛇头方向
		self.snakes = [(0, 0)] * 2  # 蛇数组
		self.isfood = True  # 食物判定
		self.bombs = []  # 炸弹数组
		self.food_pos = (randint(0, PLAYGROUND_WIDTH - 1),
		                 randint(0, PLAYGROUND_HEIGHT - 1))  # 食物坐标
		self.ate = 0  # 食物计数
		self.deathflag = False  # 死亡判定
	
	def reset(self):
		"""
		重置游戏
		"""
		self.head_pos = [0, 0]  # 蛇头位置
		self.direction = "S"  # 上一步蛇头方向
		self.snakes = [(0, 0)] * 2  # 蛇数组
		self.isfood = True  # 食物判定
		self.bombs = []  # 炸弹数组
		self.food_pos = (randint(0, PLAYGROUND_WIDTH - 1),
		                 randint(0, PLAYGROUND_HEIGHT - 1))  # 食物坐标
		self.ate = 0  # 食物计数
		self.deathflag = False  # 死亡判定
	
	def next(self, direction):
		"""
		游戏进行一步
		:param direction: 每一步的方向
		:return: 返回详细信息
		"""
		
		if direction == "W" and self.direction != "S":
			self.direction = "W"
		if direction == "S" and self.direction != "W":
			self.direction = "S"
		if direction == "A" and self.direction != "D":
			self.direction = "A"
		if direction == "D" and self.direction != "A":
			self.direction = "D"
		if self.direction == "W":
			self.head_pos[1] -= 1
		if self.direction == "S":
			self.head_pos[1] += 1
		if self.direction == "A":
			self.head_pos[0] -= 1
		if self.direction == "D":
			self.head_pos[0] += 1
		if self.head_pos[0] < 0:  # 碰到屏幕边缘循环
			self.head_pos[0] = PLAYGROUND_WIDTH - 1
		if self.head_pos[0] > PLAYGROUND_WIDTH - 1:
			self.head_pos[0] = 0
		if self.head_pos[1] < 0:
			self.head_pos[1] = PLAYGROUND_HEIGHT - 1
		if self.head_pos[1] > PLAYGROUND_HEIGHT - 1:
			self.head_pos[1] = 0
		del (self.snakes[0])  # 删除蛇数组顶
		if (self.head_pos[0], self.head_pos[1]) in self.snakes:  # 自身碰撞检测
			self.deathflag = True  # 触发死亡判定
		self.snakes.append((self.head_pos[0], self.head_pos[1]))  # 推入蛇数组底
		while len(self.bombs) < self.bomb_number:  # 刷新炸弹
			bomb_pos = (randint(0, PLAYGROUND_WIDTH - 1), randint(0, PLAYGROUND_HEIGHT - 1))
			while (bomb_pos in self.snakes) or (self.food_pos[0] == bomb_pos[0] and self.food_pos[1] == bomb_pos[1]) or (
							  bomb_pos in self.bombs):  # 避免与蛇和食物和其他炸弹重叠刷新
				bomb_pos = (randint(0, PLAYGROUND_WIDTH - 1), randint(0, PLAYGROUND_HEIGHT - 1))
			self.bombs.append(bomb_pos)
		if (self.head_pos[0], self.head_pos[1]) == self.food_pos:  # 吃食物事件
			self.snakes.append((self.head_pos[0], self.head_pos[1]))  # 将当前位置推入蛇数组
			self.ate += 1
			self.isfood = False
		if not self.isfood:  # 根据食物判定刷新食物
			self.food_pos = (randint(0, PLAYGROUND_WIDTH - 1), randint(0, PLAYGROUND_HEIGHT - 1))
			while self.food_pos in self.snakes or self.food_pos in self.bombs:  # 避免重叠刷新
				self.food_pos = (randint(0, PLAYGROUND_WIDTH - 1),
				                 randint(0, PLAYGROUND_HEIGHT - 1))
			self.isfood = True
		if (self.head_pos[0], self.head_pos[1]) in self.bombs:  # 炸弹碰撞检测
			self.deathflag = True
		return self.snakes, self.head_pos, self.food_pos, self.bombs, self.ate
	
	def get_map(self, flattening = False):
		"""
		获得游戏地图信息
		:return: 游戏地图信息，从左上到右下按行排列，将20×20的地图扁平化为400个地图数据。
				地图块上是蛇则标志位1，食物为2，炸弹为3，空为0，蛇与食物重叠则为-1。
		"""
		
		game_map = np.full((20, 20), 0)
		for S in self.snakes:
			game_map[S[1]][S[0]] = 1
		for B in self.bombs:
			game_map[B[1]][B[0]] = 3
		if self.head_pos == self.food_pos:
			game_map[self.food_pos[1]][self.food_pos[0]] = -1
		else:
			game_map[self.food_pos[1]][self.food_pos[0]] = 2
		if flattening:
			return game_map.reshape((400,))
		else:
			return game_map


#!/usr/bin/env python3

__author__ = "Yxzh"

import numpy as np
from DRL_Snakey.agent import Agent
from random import randint


class MC(Agent):
	def __init__(self, discount = 1, iteration = 20):
		self.discount = discount
		self.iteration = iteration
		
	def custom_function(self, head_pos, food_pos, snakes, now_direction):
		pass
	
	def get_next_direction(self, head_pos, food_pos, snakes, now_direction):
		pass
	



#!/usr/bin/env python3

__author__ = "Yxzh"

import random
from DRL_Snakey.agent import Agent


class Logic(Agent):
	def next(self, direction, pos):  # 预测下一步位置
		x = pos[0]
		y = pos[1]
		PLAYGROUND_WIDTH = 20
		PLAYGROUND_HEIGHT = 20  # 游戏区域大小
		if direction == "W":
			y -= 1
		if direction == "S":
			y += 1
		if direction == "A":
			x -= 1
		if direction == "D":
			x += 1
		if x < 0:
			x = PLAYGROUND_WIDTH - 1
		if x > PLAYGROUND_WIDTH - 1:
			x = 0
		if y < 0:
			y = PLAYGROUND_HEIGHT - 1
		if y > PLAYGROUND_HEIGHT - 1:
			y = 0
		return x, y
	
	def elude(self, pos, snakes):  # 躲避
		l = ["W", "S", "A", "D"]
		if self.next(l[0], pos) not in snakes:
			temp = l[0]
		elif self.next(l[1], pos) not in snakes:
			temp = l[1]
		elif self.next(l[2], pos) not in snakes:
			temp = l[2]
		elif self.next(l[3], pos) not in snakes:
			temp = l[3]
		else:
			temp = l[random.randrange(0, 4)]
		return temp
	
	def get_next_direction(self, Game):
		"""
		根据环境，通过决策算法返回前进的方向。这是决定策略类中必不可少的一个函数。
		:param Game: 游戏
		:return: 决定的前进方向
		"""
		Hs = Game.head_pos[0] - Game.food_pos[0]  # 横向差值
		Vs = Game.head_pos[1] - Game.food_pos[1]  # 纵向差值
		
		if abs(Hs) > abs(Vs):
			if Hs < 0:
				temp = "D"
				if self.next(temp, Game.head_pos) in Game.snakes:
					return self.elude(Game.head_pos, Game.snakes)[0]
			else:
				temp = "A"
				if self.next(temp, Game.head_pos) in Game.snakes:
					return self.elude(Game.head_pos, Game.snakes)[0]
		else:
			if Vs < 0:
				temp = "S"
				if self.next(temp, Game.head_pos) in Game.snakes:
					return self.elude(Game.head_pos, Game.snakes)[0]
			else:
				temp = "W"
				if self.next(temp, Game.head_pos) in Game.snakes:
					return self.elude(Game.head_pos, Game.snakes)[0]
		return temp



#!/usr/bin/env python3

__author__ = "Yxzh"

import DRL_Snakey.core


class Agent(object):
	def get_next_direction(self, Game: DRL_Snakey.core.Game):
		"""
		根据智能体对当前环境的判断选择下一步前进的方向。
		:return: 方向["W", "S", "A", "D"]
		"""
		pass
	
	def custom_function(self, Game: DRL_Snakey.core.Game):
		"""
		给不同的agnet预留的自定义函数，用来调试或数据可视化。
		"""
		pass
	
	# def visual_mode(self):
	def __str__(self):
		return "agent."
	
from .logic_AI import Logic
from .DP_AI import DP
from .MC_AI import MC


#!/usr/bin/env python3

__author__ = "Yxzh"

from DRL_Snakey.agent import Agent
import numpy as np
import matplotlib.pyplot as plt


class DP(Agent):
	"""
	Dynamic Programming 动态规划-马尔科夫决策法
	通过迭代贝尔曼方程，每一步刷新地图各个点的价值，并且朝着价值最高的点前进。
	"""
	
	def __init__(self, discount = 1, iteration = 20, walk_reward = -1, eat_self_reward = -3, food_reward = 1):
		"""
		构造DP智能体类。
		:param discount: 衰减率，贝尔曼方程中对于非即时回报的衰减率。
		:param iteration: 推算价值矩阵的迭代次数。
		:param walk_reward: 每走一步的回报
		:param eat_self_reward: 吃到自己的回报
		:param food_reward: 吃到食物的回报
		"""
		self.discount = discount
		self.strategy = np.zeros((20, 20))
		self.range = (20, 20)
		self.iteration = iteration
		self.walk_reward = walk_reward
		self.eat_self_reward = eat_self_reward
		self.food_reward = food_reward
	
	def frash_state_value(self, Game):
		"""
		进行一次价值矩阵的计算迭代。
		:param Game: 游戏
		"""
		
		game_map = Game.get_map(False)
		temp_action = ["W", "S", "A", "D"]
		temp_action.remove(self.get_opposite_direction(Game.direction))  # 排除无效动作，贪吃蛇游戏中是与前进方向相反的动作。
		for _ in range(0, self.iteration):
			temp_strategy = np.copy(self.strategy)
			for i in range(0, self.range[0]):
				for j in range(0, self.range[1]):
					temp = 0
					for a in temp_action:
						reward = self.walk_reward  # 每走一步默认回报
						x, y = self.next(a, (i, j))
						if (x, y) in Game.snakes[: -1]:
							reward = self.eat_self_reward  # 下一步吃到自己的回报
						if (x, y) == Game.food_pos:
							reward = self.food_reward  # 下一步吃到食物的回报
						temp += 1 / len(temp_action) * (reward + self.discount * self.strategy[x][y])
					temp_strategy[i][j] = temp
			temp_strategy[Game.food_pos[0]][Game.food_pos[1]] = 0  # 食物点价值最高
			self.strategy = temp_strategy
	
	def custom_function(self, Game):
		"""
		Agent类成员函数，保有的自定义函数。这里为输出价值矩阵的可视化图。此函数会重新计算价值矩阵。
		:param Game: 游戏
		"""
		self.strategy = np.zeros((20, 20))
		for _ in range(0, self.iteration):
			self.frash_state_value(Game)
		plt.matshow(self.strategy.T)
		plt.colorbar()
		plt.title(str(Game.food_pos))
		plt.show()
	
	def next(self, direction, pos):
		"""
		根据方向返回下一步时点的坐标。
		:param direction: 方向
		:param pos: 当前坐标
		:return:
		"""
		x = pos[0]
		y = pos[1]
		if direction == "W":
			y -= 1
		if direction == "S":
			y += 1
		if direction == "A":
			x -= 1
		if direction == "D":
			x += 1
		if x < 0:
			x = self.range[0] - 1
		if x > self.range[0] - 1:
			x = 0
		if y < 0:
			y = self.range[1] - 1
		if y > self.range[1] - 1:
			y = 0
		return x, y
	
	@staticmethod
	def get_opposite_direction(now_direction):
		"""
		返回相对方向。用于排除无效的动作。（向上走时不能直接向下走）
		:param now_direction: 当前方向
		:return: 无效方向
		"""
		if now_direction == "W":
			return "S"
		if now_direction == "S":
			return "W"
		if now_direction == "A":
			return "D"
		if now_direction == "D":
			return "A"
	
	def get_next_direction(self, Game):
		"""
		Agent类成员函数，根据当前环境进行决策并返回前进方向。
		:param Game: 游戏
		:return: 决策方向
		"""
		self.strategy = np.zeros((20, 20))
		self.frash_state_value(Game)
		temp_action = ["W", "S", "A", "D"]
		temp_action.remove(self.get_opposite_direction(Game.direction))
		value = {}
		for a in temp_action:
			value[a] = self.strategy[self.next(a, Game.head_pos)]
		optimal_solution = max(value, key = value.get)
		return optimal_solution



#!/usr/bin/env python3

__author__ = "Yxzh"

from setuptools import setup, find_packages


with open("README.md", "r") as fh:
	long_description = fh.read()
with open('requirements.txt') as f:
	requirements = [l for l in f.read().splitlines() if l]
setup(
	name = "DRL_Snakey",
	version = "1.0",
	description = "A Deep Reinforcement Learning study package. With game environment.",
	long_description = long_description,
	long_description_content_type = "text/markdown",
	license = "GPL",
	install_requires = requirements,
	author = "YANG Xuezhi",
	author_email = "cstrikest@gmail.com",
	url = "https://github.com/cstrikest/DRL_Snakey",
	packages = find_packages(),
	zip_safe = True,
	classifiers = [
		"Programming Language :: Python :: 3.6",
		"Environment :: MacOS X",
		"License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
		"Natural Language :: Chinese (Simplified)",
		"Topic :: Scientific/Engineering :: Artificial Intelligence",
	],
)
